import ast
import sys
from typing import Any, Literal

FIELD_MAPS: dict[type[Any] | None, dict[str, Any]] = {
    None: {},
    str: {},
    int: {},
    object: {},
    bytes: {},
    float: {},
    complex: {},
    ast.AST: {},
    ast.mod: {},
    ast.Module: {"body": list[ast.stmt], "type_ignores": list[ast.TypeIgnore]},
    ast.Interactive: {"body": list[ast.stmt]},
    ast.Expression: {"body": ast.expr},
    ast.FunctionType: {"argtypes": list[ast.expr], "returns": ast.expr},
    ast.stmt: {},
    ast.FunctionDef: {
        "name": str,
        "args": ast.arguments,
        "body": list[ast.stmt],
        "decorator_list": list[ast.expr],
        "returns": ast.expr | None,
        "type_comment": str | None,
        "type_params": list[ast.type_param],
    },
    ast.AsyncFunctionDef: {
        "name": str,
        "args": ast.arguments,
        "body": list[ast.stmt],
        "decorator_list": list[ast.expr],
        "returns": ast.expr | None,
        "type_comment": str | None,
        "type_params": list[ast.type_param],
    },
    ast.ClassDef: {
        "name": str,
        "bases": list[ast.expr],
        "keywords": list[ast.keyword],
        "body": list[ast.stmt],
        "decorator_list": list[ast.expr],
        "type_params": list[ast.type_param],
    },
    ast.Return: {"value": ast.expr | None},
    ast.Delete: {"targets": list[ast.expr]},
    ast.Assign: {"targets": list[ast.expr], "value": ast.expr, "type_comment": str | None},
    ast.TypeAlias: {"name": ast.Name, "type_params": list[ast.type_param], "value": ast.expr},
    ast.AugAssign: {"target": ast.Name | ast.Attribute | ast.Subscript, "op": ast.operator, "value": ast.expr},
    ast.AnnAssign: {
        "target": ast.Name | ast.Attribute | ast.Subscript,
        "annotation": ast.expr,
        "value": ast.expr | None,
        "simple": int,
    },
    ast.For: {
        "target": ast.expr,
        "iter": ast.expr,
        "body": list[ast.stmt],
        "orelse": list[ast.stmt],
        "type_comment": str | None,
    },
    ast.AsyncFor: {
        "target": ast.expr,
        "iter": ast.expr,
        "body": list[ast.stmt],
        "orelse": list[ast.stmt],
        "type_comment": str | None,
    },
    ast.While: {"test": ast.expr, "body": list[ast.stmt], "orelse": list[ast.stmt]},
    ast.If: {"test": ast.expr, "body": list[ast.stmt], "orelse": list[ast.stmt]},
    ast.With: {"items": list[ast.withitem], "body": list[ast.stmt], "type_comment": str | None},
    ast.AsyncWith: {"items": list[ast.withitem], "body": list[ast.stmt], "type_comment": str | None},
    ast.Match: {"subject": ast.expr, "cases": list[ast.match_case]},
    ast.Raise: {"exc": ast.expr | None, "cause": ast.expr | None},
    ast.Try: {
        "body": list[ast.stmt],
        "handlers": list[ast.ExceptHandler],
        "orelse": list[ast.stmt],
        "finalbody": list[ast.stmt],
    },
    ast.TryStar: {
        "body": list[ast.stmt],
        "handlers": list[ast.ExceptHandler],
        "orelse": list[ast.stmt],
        "finalbody": list[ast.stmt],
    },
    ast.Assert: {"test": ast.expr, "msg": ast.expr | None},
    ast.Import: {"names": list[ast.alias]},
    ast.ImportFrom: {"module": str | None, "names": list[ast.alias], "level": int},
    ast.Global: {"names": list[str]},
    ast.Nonlocal: {"names": list[str]},
    ast.Expr: {"value": ast.expr},
    ast.Pass: {},
    ast.Break: {},
    ast.Continue: {},
    ast.expr: {},
    ast.BoolOp: {"op": ast.boolop, "values": list[ast.expr]},
    ast.NamedExpr: {"target": ast.Name, "value": ast.expr},
    ast.BinOp: {"left": ast.expr, "op": ast.operator, "right": ast.expr},
    ast.UnaryOp: {"op": ast.unaryop, "operand": ast.expr},
    ast.Lambda: {"args": ast.arguments, "body": ast.expr},
    ast.IfExp: {"test": ast.expr, "body": ast.expr, "orelse": ast.expr},
    ast.Dict: {"keys": list[ast.expr | None], "values": list[ast.expr]},
    ast.Set: {"elts": list[ast.expr]},
    ast.ListComp: {"elt": ast.expr, "generators": list[ast.comprehension]},
    ast.SetComp: {"elt": ast.expr, "generators": list[ast.comprehension]},
    ast.DictComp: {"key": ast.expr, "value": ast.expr, "generators": list[ast.comprehension]},
    ast.GeneratorExp: {"elt": ast.expr, "generators": list[ast.comprehension]},
    ast.Await: {"value": ast.expr},
    ast.Yield: {"value": ast.expr | None},
    ast.YieldFrom: {"value": ast.expr},
    ast.Compare: {"left": ast.expr, "ops": list[ast.cmpop], "comparators": list[ast.expr]},
    ast.Call: {"func": ast.expr, "args": list[ast.expr], "keywords": list[ast.keyword]},
    ast.FormattedValue: {"value": ast.expr, "conversion": int, "format_spec": ast.expr | None},
    ast.JoinedStr: {"values": list[ast.expr]},
    ast.Constant: {"value": object, "kind": str | None},
    ast.Attribute: {"value": ast.expr, "attr": str},
    ast.Subscript: {"value": ast.expr, "slice": ast.expr},
    ast.Starred: {"value": ast.expr},
    ast.Name: {"id": str},
    ast.List: {"elts": list[ast.expr]},
    ast.Tuple: {"elts": list[ast.expr], "dims": list[ast.expr]},
    ast.Slice: {"lower": ast.expr | None, "upper": ast.expr | None, "step": ast.expr | None},
    ast.expr_context: {},
    ast.Load: {},
    ast.Store: {},
    ast.Del: {},
    ast.boolop: {},
    ast.And: {},
    ast.Or: {},
    ast.operator: {},
    ast.Add: {},
    ast.Sub: {},
    ast.Mult: {},
    ast.MatMult: {},
    ast.Div: {},
    ast.Mod: {},
    ast.Pow: {},
    ast.LShift: {},
    ast.RShift: {},
    ast.BitOr: {},
    ast.BitXor: {},
    ast.BitAnd: {},
    ast.FloorDiv: {},
    ast.unaryop: {},
    ast.Invert: {},
    ast.Not: {},
    ast.UAdd: {},
    ast.USub: {},
    ast.cmpop: {},
    ast.Eq: {},
    ast.NotEq: {},
    ast.Lt: {},
    ast.LtE: {},
    ast.Gt: {},
    ast.GtE: {},
    ast.Is: {},
    ast.IsNot: {},
    ast.In: {},
    ast.NotIn: {},
    ast.comprehension: {"target": ast.expr, "iter": ast.expr, "ifs": list[ast.expr], "is_async": int},
    ast.excepthandler: {},
    ast.ExceptHandler: {"type": ast.expr | None, "name": str | None, "body": list[ast.stmt]},
    ast.arguments: {
        "posonlyargs": list[ast.arg],
        "args": list[ast.arg],
        "vararg": ast.arg | None,
        "kwonlyargs": list[ast.arg],
        "kw_defaults": list[ast.expr | None],
        "kwarg": ast.arg | None,
        "defaults": list[ast.expr],
    },
    ast.arg: {"arg": str, "annotation": ast.expr | None, "type_comment": str | None},
    ast.keyword: {"arg": str | None, "value": ast.expr},
    ast.alias: {"name": str, "asname": str | None},
    ast.withitem: {"context_expr": ast.expr, "optional_vars": ast.expr | None},
    ast.match_case: {"pattern": ast.pattern, "guard": ast.expr | None, "body": list[ast.stmt]},
    ast.pattern: {},
    ast.MatchValue: {"value": ast.expr},
    ast.MatchSingleton: {"value": Literal[True, False] | None},
    ast.MatchSequence: {"patterns": list[ast.pattern]},
    ast.MatchMapping: {"keys": list[ast.expr], "patterns": list[ast.pattern], "rest": str | None},
    ast.MatchClass: {
        "cls": ast.expr,
        "patterns": list[ast.pattern],
        "kwd_attrs": list[str],
        "kwd_patterns": list[ast.pattern],
    },
    ast.MatchStar: {"name": str | None},
    ast.MatchAs: {"pattern": ast.pattern | None, "name": str | None},
    ast.MatchOr: {"patterns": list[ast.pattern]},
    ast.type_ignore: {},
    ast.TypeIgnore: {"tag": str},
    ast.type_param: {},
    ast.TypeVar: {"name": str, "bound": ast.expr | None},
    ast.ParamSpec: {"name": str},
    ast.TypeVarTuple: {"name": str},
}
if sys.version_info >= (3, 14):
    FIELD_MAPS |= {
        ast.Interpolation: {"value": ast.expr, "str": object, "conversion": int, "format_spec": ast.expr | None},
        ast.TemplateStr: {"values": list[ast.expr]},
    }
